"""Tests for launching a new server customised for juju using Nova"""

from twisted.internet import (
    defer,
    )

from juju import errors
from juju.lib import (
    mocker,
    testing,
    )
from juju.machine import (
    constraints,
    )

from juju.providers.openstack import (
    launch,
    tests,
    )


class MockedLaunchProvider(tests.MockedProvider):

    def launch(self, machine_id, master=False):
        constraint_set = constraints.ConstraintSet(self.provider_type)
        details = {
            'machine-id': machine_id,
            'constraints': constraint_set.load({}),
            }
        return launch.NovaLaunchMachine.launch(self, details, master)

    def expect_launch_setup(self, machine_id):
        self.port_manager.ensure_groups(machine_id)
        self.mocker.result(defer.succeed(["juju-x", "juju-y"]))
        self.nova.list_flavors()
        self.mocker.result(defer.succeed([
                {'id': 1, 'name': "standard.xsmall"},
            ]))

    def expect_run_server(self, machine_id, response):
        self.nova.run_server(
            name="juju testing instance " + machine_id,
            image_id=42,
            flavor_id=1,
            security_group_names=["juju-x", "juju-y"],
            user_data=mocker.ANY,
            )
        self.mocker.result(defer.succeed(response))

    def expect_available_floating_ip(self, server_id):
        self.nova.list_floating_ips()
        self.mocker.result(defer.succeed([
            {'instance_id': None, 'ip': "198.162.1.0"},
            ]))
        self.nova.add_floating_ip(server_id, "198.162.1.0")
        self.mocker.result(defer.succeed(None))


class NovaLaunchMachineTests(testing.TestCase):

    def test_launch_requires_default_image_id(self):
        config = dict(MockedLaunchProvider.default_config)
        del config['default-image-id']
        provider = MockedLaunchProvider(self.mocker, config)
        provider.expect_zookeeper_machines(1000)
        self.mocker.replay()
        deferred = provider.launch("1")
        return self.assertFailure(deferred, errors.ProviderError)

    def test_start_machine(self):
        provider = MockedLaunchProvider(self.mocker)
        provider.expect_zookeeper_machines(1000)
        provider.expect_launch_setup("1")
        provider.expect_run_server("1", response={
            'id': 1001,
            'addresses': {'public': []},
            })
        self.mocker.replay()
        return provider.launch("1")

    def test_start_machine_delay(self):
        provider = MockedLaunchProvider(self.mocker)
        provider.config["use-floating-ip"] = True
        provider.expect_zookeeper_machines(1000)
        provider.expect_launch_setup("1")
        provider.expect_run_server("1", response={
            'id': 1001,
            })
        provider.nova.get_server(1001)
        self.mocker.result(defer.succeed({
            'id': 1001,
            'addresses': {'public': []},
            }))
        provider.expect_available_floating_ip(1001)
        self.mocker.result(defer.succeed(None))
        self.mocker.replay()
        self.patch(launch.NovaLaunchMachine, "_DELAY_FOR_ADDRESSES", 0)
        return provider.launch("1")

    def test_start_machine_master(self):
        provider = MockedLaunchProvider(self.mocker)
        provider.expect_swift_public_object_url("juju_master_id")
        provider.expect_launch_setup("0")
        provider.expect_run_server("0", response={
            'id': 1000,
            'addresses': {'public': []},
            })
        provider.expect_swift_put("juju_master_id", "1000")
        provider.provider_actions.save_state({'zookeeper-instances': [1000]})
        self.mocker.result(defer.succeed(None))
        self.mocker.replay()
        return provider.launch("0", master=True)
